/*
 * Sample code for opening and writing to a .txt file in an SD card
 *
 * Author: Vaggelis Aggelou
 * Date: 17-12-2015
 */

#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "driverlib/fpu.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "driverlib/sysctl.h"
#include "driverlib/systick.h"
#include "driverlib/uart.h"
#include "uartstdio.h"
#include "ff.h"
#include "pinout.h"

void ConfigureUARTSPI(uint32_t systemClock)
{
	//Peripheral pin configuration
	PinoutSet();

	//UART0 clock setting
	UARTClockSourceSet(UART0_BASE, UART_CLOCK_PIOSC);
	UARTStdioConfig(0, 115200, systemClock);

	UARTprintf("microSD Sample Project Started! \n");
}

int main(void)
{
	FRESULT iFResult;
	FATFS sdVolume;
	FIL logFile;
	uint32_t systemClock;
	UINT count;

	systemClock = MAP_SysCtlClockFreqSet((SYSCTL_XTAL_25MHZ|SYSCTL_OSC_MAIN|
							SYSCTL_USE_PLL|SYSCTL_CFG_VCO_480), 120000000);

	// Configure SysTick for a 100Hz interrupt.
	ROM_SysTickPeriodSet(systemClock / 100);
	ROM_SysTickEnable();
	ROM_SysTickIntEnable();

	// Enable interrupts
	ROM_IntMasterEnable();

	ConfigureUARTSPI(systemClock);

	iFResult = f_mount(0, &sdVolume);
	if(iFResult != FR_OK)
	{
		UARTprintf("\n\n DEVICE NOT MOUNTED \n\n");
		return(1);
	}
	else
	{
		UARTprintf("SD card succesfully mounted! \n");
	}

	iFResult = f_open(&logFile, "sd_card_test_1.txt", FA_CREATE_NEW|FA_WRITE);
	if(iFResult != FR_OK)
	{
		UARTprintf("\n\n COULD NOT OPEN FILE \n\n");
		return(1);
	}
	else
	{
		UARTprintf("File opened succesfully");
	}

	int i = 0;
	for(i=0; i<10; i++)
	{
		iFResult = f_write(&logFile, "SPAM\n", 5, &count);
		if(iFResult != FR_OK)
		{
			UARTprintf("COULD NOT WRITE - %u\n", i);
		}
		else
		{
			UARTprintf("The size of the file is: %u\n", logFile.fsize);
		}
	}

	f_close(&logFile);
	f_mount(0, NULL);

	return 0;
}
